# Copyright 2011, 2012, 2013, 2014 Kevin Ryde

# Generated by Math-NumSeq tools/make-oeis-catalogue.pl -- DO NOT EDIT

# This file is part of Math-PlanePath.
#
# Math-PlanePath is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# Math-PlanePath is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with Math-PlanePath.  If not, see <http://www.gnu.org/licenses/>.

package Math::NumSeq::OEIS::Catalogue::Plugin::PlanePath;
use 5.004;
use strict;

use vars '$VERSION', '@ISA';
$VERSION = 129;
use Math::NumSeq::OEIS::Catalogue::Plugin;
@ISA = ('Math::NumSeq::OEIS::Catalogue::Plugin');

## no critic (CodeLayout::RequireTrailingCommaAtNewline)

# total 300 A-numbers in 4 modules

use constant info_arrayref =>
[
  {
    'anum' => 'A174344',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'SquareSpiral',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A214526',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'SquareSpiral',
      'coordinate_type',
      'SumAbs'
    ]
  },
  {
    'anum' => 'A336336',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'SquareSpiral',
      'coordinate_type',
      'RSquared'
    ]
  },
  {
    'anum' => 'A180714',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'SquareSpiral,n_start=0',
      'coordinate_type',
      'Sum'
    ]
  },
  {
    'anum' => 'A329116',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PyramidSpiral,n_start=0',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A329972',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PyramidSpiral,n_start=0',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A053615',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PyramidSpiral,n_start=0',
      'coordinate_type',
      'ExperimentalAbsX'
    ]
  },
  {
    'anum' => 'A010751',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'DiamondSpiral,n_start=0'
    ]
  },
  {
    'anum' => 'A305258',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'DiamondSpiral,n_start=0',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A328818',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'HexSpiral,n_start=0',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A307012',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'HexSpiral,n_start=0',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A321768',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PythagoreanTree',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A321769',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PythagoreanTree',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A321770',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PythagoreanTree,coordinates=AC',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A321782',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PythagoreanTree,coordinates=PQ',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A321783',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PythagoreanTree,coordinates=PQ',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A321784',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PythagoreanTree,coordinates=PQ',
      'coordinate_type',
      'Sum'
    ]
  },
  {
    'anum' => 'A321785',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PythagoreanTree,coordinates=PQ',
      'coordinate_type',
      'DiffXY'
    ]
  },
  {
    'anum' => 'A153036',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'RationalsTree',
      'coordinate_type',
      'IntXY'
    ]
  },
  {
    'anum' => 'A000523',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'RationalsTree',
      'coordinate_type',
      'Depth'
    ]
  },
  {
    'anum' => 'A070871',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'RationalsTree,tree_type=CW',
      'coordinate_type',
      'Product'
    ]
  },
  {
    'anum' => 'A020650',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'RationalsTree,tree_type=AYT',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A020651',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'RationalsTree,tree_type=AYT',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A135523',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'RationalsTree,tree_type=AYT',
      'coordinate_type',
      'IntXY'
    ]
  },
  {
    'anum' => 'A162909',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'RationalsTree,tree_type=Bird',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A162910',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'RationalsTree,tree_type=Bird',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A162911',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'RationalsTree,tree_type=Drib',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A162912',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'RationalsTree,tree_type=Drib',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A174981',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'RationalsTree,tree_type=L',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A086592',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'FractionsTree',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A191379',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'ChanTree'
    ]
  },
  {
    'anum' => 'A163528',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PeanoCurve',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A163529',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PeanoCurve',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A163530',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PeanoCurve',
      'coordinate_type',
      'Sum'
    ]
  },
  {
    'anum' => 'A163531',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PeanoCurve',
      'coordinate_type',
      'RSquared'
    ]
  },
  {
    'anum' => 'A059253',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'HilbertCurve',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A059252',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'HilbertCurve',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A059261',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'HilbertCurve',
      'coordinate_type',
      'Sum'
    ]
  },
  {
    'anum' => 'A059285',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'HilbertCurve',
      'coordinate_type',
      'DiffXY'
    ]
  },
  {
    'anum' => 'A163547',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'HilbertCurve',
      'coordinate_type',
      'RSquared'
    ]
  },
  {
    'anum' => 'A059905',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'ZOrderCurve',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A059906',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'ZOrderCurve',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A292373',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'ZOrderCurve',
      'coordinate_type',
      'BitAnd'
    ]
  },
  {
    'anum' => 'A309952',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'ZOrderCurve',
      'coordinate_type',
      'BitXor'
    ]
  },
  {
    'anum' => 'A163325',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'ZOrderCurve,radix=3',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A163326',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'ZOrderCurve,radix=3',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A080463',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'ZOrderCurve,radix=10',
      'coordinate_type',
      'Sum'
    ]
  },
  {
    'anum' => 'A080464',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'ZOrderCurve,radix=10',
      'coordinate_type',
      'Product',
      'i_start',
      10
    ]
  },
  {
    'anum' => 'A080465',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'ZOrderCurve,radix=10',
      'coordinate_type',
      'AbsDiff',
      'i_start',
      10
    ]
  },
  {
    'anum' => 'A334486',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Flowsnake',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A332246',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'QuadricCurve',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A332247',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'QuadricCurve',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A334483',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'SierpinskiArrowhead,align=diagonal',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A334484',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'SierpinskiArrowhead,align=diagonal',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A334235',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'HIndexing',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A334236',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'HIndexing',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A332383',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'DragonCurve',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A332384',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'DragonCurve',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A334576',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'AlternatePaperMidpoint',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A334577',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'AlternatePaperMidpoint',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A332251',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'CCurve',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A332252',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'CCurve',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A318438',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'ComplexMinus',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A318439',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'ComplexMinus',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A318479',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'ComplexMinus',
      'coordinate_type',
      'RSquared'
    ]
  },
  {
    'anum' => 'A057554',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals',
      'coordinate_type',
      'ExperimentalPairsXY'
    ]
  },
  {
    'anum' => 'A164306',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,x_start=1,y_start=0',
      'coordinate_type',
      'ExperimentalNumerator'
    ]
  },
  {
    'anum' => 'A167192',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,x_start=1,y_start=0',
      'coordinate_type',
      'ExperimentalDenominator'
    ]
  },
  {
    'anum' => 'A003991',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,x_start=1,y_start=1',
      'coordinate_type',
      'Product'
    ]
  },
  {
    'anum' => 'A003989',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,x_start=1,y_start=1',
      'coordinate_type',
      'GCD'
    ]
  },
  {
    'anum' => 'A003983',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,x_start=1,y_start=1',
      'coordinate_type',
      'Min'
    ]
  },
  {
    'anum' => 'A051125',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,x_start=1,y_start=1',
      'coordinate_type',
      'Max'
    ]
  },
  {
    'anum' => 'A004199',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,x_start=1,y_start=1',
      'coordinate_type',
      'IntXY'
    ]
  },
  {
    'anum' => 'A057555',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,x_start=1,y_start=1',
      'coordinate_type',
      'ExperimentalPairsXY'
    ]
  },
  {
    'anum' => 'A003988',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,direction=up,x_start=1,y_start=1',
      'coordinate_type',
      'IntXY'
    ]
  },
  {
    'anum' => 'A112543',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,direction=up,x_start=1,y_start=1',
      'coordinate_type',
      'ExperimentalNumerator'
    ]
  },
  {
    'anum' => 'A112544',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,direction=up,x_start=1,y_start=1',
      'coordinate_type',
      'ExperimentalDenominator'
    ]
  },
  {
    'anum' => 'A004247',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,n_start=0',
      'coordinate_type',
      'Product'
    ]
  },
  {
    'anum' => 'A114327',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,n_start=0',
      'coordinate_type',
      'DiffYX'
    ]
  },
  {
    'anum' => 'A049581',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,n_start=0',
      'coordinate_type',
      'AbsDiff'
    ]
  },
  {
    'anum' => 'A048147',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,n_start=0',
      'coordinate_type',
      'RSquared'
    ]
  },
  {
    'anum' => 'A004198',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,n_start=0',
      'coordinate_type',
      'BitAnd'
    ]
  },
  {
    'anum' => 'A003986',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,n_start=0',
      'coordinate_type',
      'BitOr'
    ]
  },
  {
    'anum' => 'A003987',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,n_start=0',
      'coordinate_type',
      'BitXor'
    ]
  },
  {
    'anum' => 'A109004',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,n_start=0',
      'coordinate_type',
      'GCD'
    ]
  },
  {
    'anum' => 'A004197',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,n_start=0',
      'coordinate_type',
      'Min'
    ]
  },
  {
    'anum' => 'A003984',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,n_start=0',
      'coordinate_type',
      'Max'
    ]
  },
  {
    'anum' => 'A101080',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Diagonals,n_start=0',
      'coordinate_type',
      'ExperimentalHammingDist'
    ]
  },
  {
    'anum' => 'A319572',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'DiagonalsAlternating,n_start=0',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A319573',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'DiagonalsAlternating,n_start=0',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A055087',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'DiagonalsOctant,n_start=0',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A055086',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'DiagonalsOctant,n_start=0',
      'coordinate_type',
      'Sum'
    ]
  },
  {
    'anum' => 'A082375',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'DiagonalsOctant,n_start=0',
      'coordinate_type',
      'DiffYX'
    ]
  },
  {
    'anum' => 'A213088',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'Corner',
      'coordinate_type',
      'Sum'
    ]
  },
  {
    'anum' => 'A319290',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'CornerAlternating,n_start=0',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A319289',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'CornerAlternating,n_start=0',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A079904',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PyramidRows,step=1,n_start=0',
      'coordinate_type',
      'Product'
    ]
  },
  {
    'anum' => 'A069011',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PyramidRows,step=1,n_start=0',
      'coordinate_type',
      'RSquared'
    ]
  },
  {
    'anum' => 'A080099',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PyramidRows,step=1,n_start=0',
      'coordinate_type',
      'BitAnd'
    ]
  },
  {
    'anum' => 'A080098',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PyramidRows,step=1,n_start=0',
      'coordinate_type',
      'BitOr'
    ]
  },
  {
    'anum' => 'A051933',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PyramidRows,step=1,n_start=0',
      'coordinate_type',
      'BitXor'
    ]
  },
  {
    'anum' => 'A196199',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PyramidRows,n_start=0',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A000196',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PyramidRows,n_start=0',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A180447',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PyramidRows,step=3,n_start=0',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A060511',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'PyramidRows,step=4,align=right,n_start=0',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A004396',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'CellularRule,rule=6,n_start=0',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A131452',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'CellularRule,rule=6,n_start=0',
      'coordinate_type',
      'SumAbs'
    ]
  },
  {
    'anum' => 'A004523',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'CellularRule,rule=20,n_start=0',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A004773',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'CellularRule,rule=20,n_start=0',
      'coordinate_type',
      'SumAbs'
    ]
  },
  {
    'anum' => 'A020652',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'DiagonalRationals',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A020653',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'DiagonalRationals',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A071974',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'FactorRationals',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A071975',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'FactorRationals',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A019554',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'FactorRationals',
      'coordinate_type',
      'Product'
    ]
  },
  {
    'anum' => 'A226314',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'GcdRationals',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A054531',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'GcdRationals',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A038567',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'CoprimeColumns',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A038566',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'CoprimeColumns,n_start=1',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A061017',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'DivisibleColumns,n_start=1',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A027750',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'DivisibleColumns,n_start=1',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A056538',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'DivisibleColumns,n_start=1',
      'coordinate_type',
      'IntXY'
    ]
  },
  {
    'anum' => 'A208460',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'DivisibleColumns,divisor_type=proper,n_start=2',
      'coordinate_type',
      'DiffXY'
    ]
  },
  {
    'anum' => 'A316657',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'QuintetReplicate',
      'coordinate_type',
      'X'
    ]
  },
  {
    'anum' => 'A316658',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'QuintetReplicate',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A316707',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'QuintetReplicate',
      'coordinate_type',
      'RSquared'
    ]
  },
  {
    'anum' => 'A139351',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'CornerReplicate',
      'coordinate_type',
      'ExperimentalHammingDist'
    ]
  },
  {
    'anum' => 'A019586',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'WythoffArray',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A191360',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'WythoffArray',
      'coordinate_type',
      'DiffXY'
    ]
  },
  {
    'anum' => 'A035612',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'WythoffArray,x_start=1,y_start=1'
    ]
  },
  {
    'anum' => 'A003603',
    'class' => 'Math::NumSeq::PlanePathCoord',
    'parameters' => [
      'planepath',
      'WythoffArray,x_start=1,y_start=1',
      'coordinate_type',
      'Y'
    ]
  },
  {
    'anum' => 'A079813',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'SquareSpiral',
      'delta_type',
      'AbsdY'
    ]
  },
  {
    'anum' => 'A339265',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'PyramidSpiral,n_start=0',
      'delta_type',
      'dX'
    ]
  },
  {
    'anum' => 'A204439',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'TriangleSpiralSkewed,skew=left',
      'delta_type',
      'AbsdX'
    ]
  },
  {
    'anum' => 'A204437',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'TriangleSpiralSkewed,skew=left',
      'delta_type',
      'AbsdY'
    ]
  },
  {
    'anum' => 'A204435',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'TriangleSpiralSkewed,skew=right',
      'delta_type',
      'AbsdX'
    ]
  },
  {
    'anum' => 'A003982',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'DiamondSpiral,n_start=0',
      'delta_type',
      'dSumAbs'
    ]
  },
  {
    'anum' => 'A023532',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'AztecDiamondRings,n_start=0',
      'delta_type',
      'AbsdY'
    ]
  },
  {
    'anum' => 'A070990',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'RationalsTree,tree_type=L',
      'delta_type',
      'dY'
    ]
  },
  {
    'anum' => 'A011655',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'KochCurve',
      'delta_type',
      'AbsdY'
    ]
  },
  {
    'anum' => 'A246960',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'DragonCurve',
      'delta_type',
      'Dir4'
    ]
  },
  {
    'anum' => 'A166486',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'DragonRounded',
      'delta_type',
      'AbsdY'
    ]
  },
  {
    'anum' => 'A152822',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'DragonRounded',
      'delta_type',
      'AbsdX'
    ]
  },
  {
    'anum' => 'A010059',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'CCurve',
      'delta_type',
      'AbsdX'
    ]
  },
  {
    'anum' => 'A033999',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'Rows,width=2,n_start=0',
      'delta_type',
      'dX'
    ]
  },
  {
    'anum' => 'A124625',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'Rows,width=2,n_start=0',
      'delta_type',
      'dRSquared'
    ]
  },
  {
    'anum' => 'A010673',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'Rows,width=2,n_start=0',
      'delta_type',
      'TDir6'
    ]
  },
  {
    'anum' => 'A061347',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'Rows,width=3',
      'delta_type',
      'dX'
    ]
  },
  {
    'anum' => 'A131561',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'Rows,width=3,n_start=0',
      'delta_type',
      'dSum'
    ]
  },
  {
    'anum' => 'A127949',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'Diagonals',
      'delta_type',
      'dY'
    ]
  },
  {
    'anum' => 'A051340',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'Diagonals,n_start=0',
      'delta_type',
      'AbsdY'
    ]
  },
  {
    'anum' => 'A023531',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'PyramidRows,step=1,n_start=0',
      'delta_type',
      'dY'
    ]
  },
  {
    'anum' => 'A049240',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'PyramidSides',
      'delta_type',
      'AbsdY'
    ]
  },
  {
    'anum' => 'A177702',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'CellularRule,rule=20,n_start=0',
      'delta_type',
      'dSumAbs'
    ]
  },
  {
    'anum' => 'A102283',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'CellularRule,rule=6,n_start=0',
      'delta_type',
      'dSum'
    ]
  },
  {
    'anum' => 'A131756',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'CellularRule,rule=6,n_start=0',
      'delta_type',
      'dSumAbs'
    ]
  },
  {
    'anum' => 'A062157',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'CellularRule,rule=14,n_start=0',
      'delta_type',
      'dSum'
    ]
  },
  {
    'anum' => 'A109613',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'CellularRule,rule=84,n_start=0',
      'delta_type',
      'dRSquared'
    ]
  },
  {
    'anum' => 'A171587',
    'class' => 'Math::NumSeq::PlanePathDelta',
    'parameters' => [
      'planepath',
      'FibonacciWordFractal',
      'delta_type',
      'AbsdX'
    ]
  },
  {
    'anum' => 'A054552',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'SquareSpiral'
    ]
  },
  {
    'anum' => 'A033951',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'SquareSpiral',
      'line_type',
      'Y_neg'
    ]
  },
  {
    'anum' => 'A053755',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'SquareSpiral',
      'line_type',
      'Diagonal_NW'
    ]
  },
  {
    'anum' => 'A016754',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'SquareSpiral',
      'line_type',
      'Diagonal_SE'
    ]
  },
  {
    'anum' => 'A033991',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'SquareSpiral,n_start=0',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A002939',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'SquareSpiral,n_start=0',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A002943',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'SquareSpiral,n_start=0',
      'line_type',
      'Diagonal_SW'
    ]
  },
  {
    'anum' => 'A069894',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'SquareSpiral,wider=1',
      'line_type',
      'Diagonal_SW'
    ]
  },
  {
    'anum' => 'A185669',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'PyramidSpiral,n_start=2',
      'line_type',
      'Diagonal_SE'
    ]
  },
  {
    'anum' => 'A062741',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'TriangleSpiral,n_start=0',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A062708',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'TriangleSpiral,n_start=0',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A062725',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'TriangleSpiral,n_start=0',
      'line_type',
      'Diagonal_SW'
    ]
  },
  {
    'anum' => 'A117625',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'TriangleSpiralSkewed'
    ]
  },
  {
    'anum' => 'A006137',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'TriangleSpiralSkewed',
      'line_type',
      'X_neg'
    ]
  },
  {
    'anum' => 'A064225',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'TriangleSpiralSkewed',
      'line_type',
      'Y_neg'
    ]
  },
  {
    'anum' => 'A081589',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'TriangleSpiralSkewed',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A038764',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'TriangleSpiralSkewed',
      'line_type',
      'Diagonal_SW'
    ]
  },
  {
    'anum' => 'A081267',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'TriangleSpiralSkewed',
      'line_type',
      'Diagonal_SE'
    ]
  },
  {
    'anum' => 'A081274',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'TriangleSpiralSkewed',
      'line_type',
      'Diagonal_SW'
    ]
  },
  {
    'anum' => 'A081266',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'TriangleSpiralSkewed,n_start=0',
      'line_type',
      'Diagonal_SW'
    ]
  },
  {
    'anum' => 'A130883',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'DiamondSpiral'
    ]
  },
  {
    'anum' => 'A058331',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'DiamondSpiral',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A192136',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'PentSpiralSkewed'
    ]
  },
  {
    'anum' => 'A116668',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'PentSpiralSkewed',
      'line_type',
      'X_neg'
    ]
  },
  {
    'anum' => 'A158187',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'PentSpiralSkewed',
      'line_type',
      'Diagonal_NW'
    ]
  },
  {
    'anum' => 'A005891',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'PentSpiralSkewed',
      'line_type',
      'Diagonal_SE'
    ]
  },
  {
    'anum' => 'A005476',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'PentSpiralSkewed,n_start=0',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A005475',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'PentSpiralSkewed,n_start=0',
      'line_type',
      'X_neg'
    ]
  },
  {
    'anum' => 'A028895',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'PentSpiralSkewed,n_start=0',
      'line_type',
      'Diagonal_SE'
    ]
  },
  {
    'anum' => 'A049450',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'HexSpiral,n_start=0',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A028896',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'HexSpiral,n_start=0',
      'line_type',
      'Diagonal_SE'
    ]
  },
  {
    'anum' => 'A056105',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'HexSpiralSkewed'
    ]
  },
  {
    'anum' => 'A056106',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'HexSpiralSkewed',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A056108',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'HexSpiralSkewed',
      'line_type',
      'X_neg'
    ]
  },
  {
    'anum' => 'A056109',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'HexSpiralSkewed',
      'line_type',
      'Y_neg'
    ]
  },
  {
    'anum' => 'A056107',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'HexSpiralSkewed',
      'line_type',
      'Diagonal_NW'
    ]
  },
  {
    'anum' => 'A049451',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'HexSpiralSkewed,n_start=0',
      'line_type',
      'X_neg'
    ]
  },
  {
    'anum' => 'A062783',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'HexSpiralSkewed,n_start=0',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A063436',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'HexSpiralSkewed,n_start=0',
      'line_type',
      'Diagonal_SW'
    ]
  },
  {
    'anum' => 'A022265',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'HeptSpiralSkewed,n_start=0',
      'line_type',
      'X_neg'
    ]
  },
  {
    'anum' => 'A218471',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'HeptSpiralSkewed,n_start=0',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A195023',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'HeptSpiralSkewed,n_start=0',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A022264',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'HeptSpiralSkewed,n_start=0',
      'line_type',
      'Diagonal_NW'
    ]
  },
  {
    'anum' => 'A186029',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'HeptSpiralSkewed,n_start=0',
      'line_type',
      'Diagonal_SW'
    ]
  },
  {
    'anum' => 'A024966',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'HeptSpiralSkewed,n_start=0',
      'line_type',
      'Diagonal_SE'
    ]
  },
  {
    'anum' => 'A139273',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'OctagramSpiral,n_start=0',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A139275',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'OctagramSpiral,n_start=0',
      'line_type',
      'X_neg'
    ]
  },
  {
    'anum' => 'A139277',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'OctagramSpiral,n_start=0',
      'line_type',
      'Y_neg'
    ]
  },
  {
    'anum' => 'A139272',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'OctagramSpiral,n_start=0',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A139274',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'OctagramSpiral,n_start=0',
      'line_type',
      'Diagonal_NW'
    ]
  },
  {
    'anum' => 'A139276',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'OctagramSpiral,n_start=0',
      'line_type',
      'Diagonal_SW'
    ]
  },
  {
    'anum' => 'A033570',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'AnvilSpiral'
    ]
  },
  {
    'anum' => 'A033568',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'AnvilSpiral',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A085473',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'AnvilSpiral',
      'line_type',
      'Diagonal_SE'
    ]
  },
  {
    'anum' => 'A126587',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'AnvilSpiral',
      'line_type',
      'Y_axis',
      'i_start',
      1
    ]
  },
  {
    'anum' => 'A139267',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'AnvilSpiral,n_start=0',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A049452',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'AnvilSpiral,n_start=0',
      'line_type',
      'X_neg'
    ]
  },
  {
    'anum' => 'A033580',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'AnvilSpiral,n_start=0',
      'line_type',
      'Y_neg'
    ]
  },
  {
    'anum' => 'A094159',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'AnvilSpiral,n_start=0',
      'line_type',
      'Diagonal_NW'
    ]
  },
  {
    'anum' => 'A049453',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'AnvilSpiral,n_start=0',
      'line_type',
      'Diagonal_SW'
    ]
  },
  {
    'anum' => 'A195319',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'AnvilSpiral,n_start=0',
      'line_type',
      'Diagonal_SE'
    ]
  },
  {
    'anum' => 'A051132',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'Hypot,n_start=0'
    ]
  },
  {
    'anum' => 'A036702',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'HypotOctant,points=even',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A007051',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'PythagoreanTree',
      'line_type',
      'Depth_start'
    ]
  },
  {
    'anum' => 'A081254',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'RationalsTree,tree_type=Bird'
    ]
  },
  {
    'anum' => 'A086893',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'RationalsTree,tree_type=Drib'
    ]
  },
  {
    'anum' => 'A102631',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'FactorRationals',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A163480',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'PeanoCurve'
    ]
  },
  {
    'anum' => 'A163481',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'PeanoCurve',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A163343',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'PeanoCurve',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A163482',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'HilbertCurve'
    ]
  },
  {
    'anum' => 'A163483',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'HilbertCurve',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A062880',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'ZOrderCurve',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A001196',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'ZOrderCurve',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A037314',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'ZOrderCurve,radix=3'
    ]
  },
  {
    'anum' => 'A338086',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'ZOrderCurve,radix=3',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A208665',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'ZOrderCurve,radix=3',
      'i_start',
      1,
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A051022',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'ZOrderCurve,radix=10'
    ]
  },
  {
    'anum' => 'A338754',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'ZOrderCurve,radix=10',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A163344',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'GrayCode,apply_type=sT,radix=3'
    ]
  },
  {
    'anum' => 'A006046',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'SierpinskiTriangle,align=diagonal,n_start=0',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A074330',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'SierpinskiTriangle',
      'line_type',
      'Diagonal',
      'i_start',
      1
    ]
  },
  {
    'anum' => 'A066321',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'ComplexMinus'
    ]
  },
  {
    'anum' => 'A256441',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'ComplexMinus',
      'line_type',
      'X_neg'
    ]
  },
  {
    'anum' => 'A016777',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'Rows,width=3',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A016813',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'Rows,width=4',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A016861',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'Rows,width=5',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A016921',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'Rows,width=6',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A016993',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'Rows,width=7',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A000124',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'Diagonals',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A001844',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'Diagonals',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A096376',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'Staircase,n_start=2',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A059100',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'Corner,n_start=2',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A014206',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'Corner,n_start=2',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A028552',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'Corner,wider=2,n_start=0',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A028387',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'Corner,wider=2,n_start=1',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A081346',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'CornerAlternating'
    ]
  },
  {
    'anum' => 'A081345',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'CornerAlternating',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A081347',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'CornerAlternating,wider=1'
    ]
  },
  {
    'anum' => 'A081348',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'CornerAlternating,wider=1',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A080335',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'CornerAlternating,wider=1',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A081350',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'CornerAlternating,wider=2'
    ]
  },
  {
    'anum' => 'A081351',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'CornerAlternating,wider=2',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A081352',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'CornerAlternating,wider=2',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A104249',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'PyramidRows,step=3',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A143689',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'PyramidRows,step=3',
      'line_type',
      'Diagonal_NW'
    ]
  },
  {
    'anum' => 'A084849',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'PyramidRows,step=4',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A046092',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'PyramidRows,step=4,n_start=0',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A002522',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'PyramidSides',
      'line_type',
      'X_neg'
    ]
  },
  {
    'anum' => 'A061925',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'CellularRule,rule=5',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A006578',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'CellularRule190,n_start=0',
      'line_type',
      'Diagonal_NW'
    ]
  },
  {
    'anum' => 'A147562',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'UlamWarburton,n_start=0',
      'line_type',
      'Depth_start'
    ]
  },
  {
    'anum' => 'A183060',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'UlamWarburton,parts=2,n_start=0',
      'line_type',
      'Depth_start'
    ]
  },
  {
    'anum' => 'A151922',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'UlamWarburton,parts=1,n_start=1',
      'line_type',
      'Depth_end'
    ]
  },
  {
    'anum' => 'A151920',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'UlamWarburtonQuarter',
      'line_type',
      'Depth_end'
    ]
  },
  {
    'anum' => 'A084471',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'DigitGroups,radix=2',
      'i_start',
      1
    ]
  },
  {
    'anum' => 'A003622',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'WythoffArray,x_start=1,y_start=1',
      'line_type',
      'Y_axis'
    ]
  },
  {
    'anum' => 'A020941',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'WythoffArray,x_start=1,y_start=1',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A014480',
    'class' => 'Math::NumSeq::PlanePathN',
    'parameters' => [
      'planepath',
      'PowerArray',
      'line_type',
      'Diagonal'
    ]
  },
  {
    'anum' => 'A163536',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'PeanoCurve',
      'turn_type',
      'SLR'
    ]
  },
  {
    'anum' => 'A163537',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'PeanoCurve',
      'turn_type',
      'SRL'
    ]
  },
  {
    'anum' => 'A163542',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'HilbertCurve',
      'turn_type',
      'SLR'
    ]
  },
  {
    'anum' => 'A163543',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'HilbertCurve',
      'turn_type',
      'SRL'
    ]
  },
  {
    'anum' => 'A035263',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'KochCurve'
    ]
  },
  {
    'anum' => 'A309873',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'KochCurve',
      'turn_type',
      'LSR'
    ]
  },
  {
    'anum' => 'A056832',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'KochCurve',
      'turn_type',
      'SLR'
    ]
  },
  {
    'anum' => 'A034947',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'DragonCurve',
      'turn_type',
      'LSR'
    ]
  },
  {
    'anum' => 'A099545',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'DragonCurve',
      'turn_type',
      'Turn4'
    ]
  },
  {
    'anum' => 'A209615',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'AlternatePaper',
      'turn_type',
      'LSR'
    ]
  },
  {
    'anum' => 'A292077',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'AlternatePaper',
      'turn_type',
      'Right'
    ]
  },
  {
    'anum' => 'A137893',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'GosperSide'
    ]
  },
  {
    'anum' => 'A060236',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'GosperSide',
      'turn_type',
      'SLR'
    ]
  },
  {
    'anum' => 'A129184',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'Diagonals,n_start=0'
    ]
  },
  {
    'anum' => 'A156319',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'Diagonals,n_start=0',
      'turn_type',
      'SRL'
    ]
  },
  {
    'anum' => 'A097806',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'Diagonals,n_start=-1',
      'turn_type',
      'NotStraight'
    ]
  },
  {
    'anum' => 'A000007',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'Corner,wider=1,n_start=-1'
    ]
  },
  {
    'anum' => 'A063524',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'Corner,wider=2,n_start=-1'
    ]
  },
  {
    'anum' => 'A185012',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'Corner,wider=3,n_start=-1'
    ]
  },
  {
    'anum' => 'A131534',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'CellularRule,rule=6,n_start=-1',
      'turn_type',
      'SRL'
    ]
  },
  {
    'anum' => 'A130196',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'CellularRule,rule=20,n_start=-1',
      'turn_type',
      'SRL'
    ]
  },
  {
    'anum' => 'A176040',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'CellularRule,rule=84,n_start=-1',
      'turn_type',
      'Turn4'
    ]
  },
  {
    'anum' => 'A156596',
    'class' => 'Math::NumSeq::PlanePathTurn',
    'parameters' => [
      'planepath',
      'FibonacciWordFractal',
      'turn_type',
      'SRL'
    ]
  }
]
;
1;
__END__
